function [bigV, bigWas] = bigVWfromData(evLength, trainData, alphabetsize)
% generating  "oversized" counting matrices V, Wa's (one Wa for each observable a) 
% bigV and bigWas are normalized to total component sum 1
% bigV and bigWas are sparse matrices
%
% input arguments:
% evLength: length of characteristic sequences
% trainData: row vector containing the training sequence, where n symbols are
% coded by integers 1,...,n
% alphabetsize: self-explaining
%
% outputs:
% bigV: a sparse matrix of size \kappa x \kappa, bigV = (\hat{P}(\bar{a}_j \bar{b}_i))
% bigWas: a structure with elements bigWas(a).W, where a ranges from 1 to
%         alphabetsize. bigWas(a).W = (\hat{P}(\bar{a}_j a \bar{b}_i))
%
% Version 1.0, May 11, 2005
% (c) Herbert Jaeger
%
% Copyright and limitation of liability: as specified in the document
% Copyright.txt that comes along with the OOM toolbox package from which 
% this file is taken.

% initialize 
k = alphabetsize^evLength; % corresponds to the \kappa in my OOM papers
bigV = sparse(k,k);
bigWas = struct('W',{});
for a = 1:alphabetsize
    bigWas(a).W = sparse(k,k);
end
bigWindow = [0 trainData(1,1:2*evLength)];
effectiveSampleLength = length(trainData) -  2*evLength;

% sample
for n = 1+2*evLength:length(trainData)
    choice = trainData(1,n);
    bigWindow = [bigWindow(1,2:end) choice];    
    indIndexBig = charIndex(bigWindow(1,1:evLength), alphabetsize);
    charIndexForBigV = charIndex(bigWindow(1,evLength+1:2*evLength), alphabetsize);
    charIndexForBigW = charIndex(bigWindow(1,evLength+2:2*evLength+1), alphabetsize);
    bigV(charIndexForBigV,indIndexBig) = bigV(charIndexForBigV,indIndexBig) + 1;
    
   bigWas(bigWindow(1,evLength+1)).W(charIndexForBigW, indIndexBig) = ...
        bigWas(bigWindow(1,evLength+1)).W(charIndexForBigW, indIndexBig) + 1;      
end

bigV = bigV / effectiveSampleLength;
for a = 1:alphabetsize
    bigWas(a).W = bigWas(a).W  / effectiveSampleLength;
end




