function bigV = bigVfromData(evLength, trainData, alphabetsize)
% generating  "oversized" counting matrix bigV (is \underline{\hat{V}} in
% Haykin book chapter)
% bigV is normalized to total component sum 1
% bigV is sparse
%
% input arguments:
% evLength: length of characteristic sequences
% trainData: row vector containing the training sequence, where n symbols are
% coded by integers 1,...,n
% alphabetsize: self-explaining
%
% output:
% bigV: a sparse matrix of size \kappa x \kappa, bigV = (\hat{P}(\bar{a}_j \bar{b}_i))
%
% Version 1.0, May 11, 2005
% (c) Herbert Jaeger
%
% Copyright and limitation of liability: as specified in the document
% Copyright.txt that comes along with the OOM toolbox package from which 
% this file is taken.

% initialize 
k = alphabetsize^evLength; % corresponds to the \kappa in my OOM papers
bigV = sparse(k,k);
bigWindow = [0 trainData(1,1:2*evLength-1)];
effectiveSampleLength = length(trainData) -  2*evLength + 1;

% sample
for n = 2*evLength:length(trainData)
    choice = trainData(1,n);
    bigWindow = [bigWindow(1,2:end) choice];    
    indIndexBig = charIndex(bigWindow(1,1:evLength), alphabetsize);
    charIndexForBigV = charIndex(bigWindow(1,evLength+1:2*evLength), alphabetsize);    
    bigV(charIndexForBigV,indIndexBig) = bigV(charIndexForBigV,indIndexBig) + 1;
end

bigV = bigV / effectiveSampleLength;




