function [bigV, PiMat, stateMat] = bigVfromModel(tl,w0,evLength)
% computes bigV = (P(\bar{a}_i \bar{b}_j)_{i,j = 1,...,\kappa}, 
% PiMat (size kappa x dim) with i-th row being \mathbf{1} \tau_{\bar{a}_i}
% stateMat (size dim x kappa) with j-th column being w_{\bar{a}_j}
%
% V 1.0 June 1
% (c) Herbert Jaeger
%
% Copyright and limitation of liability: as specified in the document
% Copyright.txt that comes along with the OOM toolbox package from which 
% this file is taken.

alphabetsize = size(tl,3);
dim = size(tl,1);
kappa = alphabetsize^evLength;
PiMat = zeros(kappa, dim); 
stateMat = zeros(dim,kappa);

for i = 1:kappa
    % generate i-th sequence seq in alphabetic enumeration
    seq = zeros(1,evLength);
    j = i-1;
    for k = evLength : - 1 : 1        
        ind =  floor((j)/alphabetsize^(k-1));
        j = j - ind * alphabetsize^(k-1);        
        seq(1,evLength-k+1) = ind + 1;
    end
    r = ones(1,dim);
    for n = evLength:-1:1
        r = r * tl(:,:,seq(n));            
    end
    PiMat(i,:) = r;
    c = w0;
    for n = 1:evLength
        c = tl(:,:,seq(n)) * c;
    end
    stateMat(:,i) = c;
end

bigV = PiMat * stateMat;
