function seq = genSeq(tl, w0, n)
% generates a random sequence seq from OOM taulist tl, start vector w0, of length n
% seq is a row vector with integer entries ranging from 1 to alphabetsize.
% The function is non-robust in the sense that it expects a valid OOM. For
% an OOM which may be invalid (ie., might predict negative probabilities
% sometimes), use compStateObsSeq_robust instead.
% 
% V 1.0, May 2005
% (c) Herbert Jaeger
%
% Copyright and limitation of liability: as specified in the document
% Copyright.txt that comes along with the OOM toolbox package from which 
% this file is taken.

alphabetsize = size(tl,3);
dim = size(tl,1);
seq = zeros(1,n);
w = w0;
squeezedTL = zeros(alphabetsize,dim);
for i = 1:alphabetsize
    squeezedTL(i,:) = sum(tl(:,:,i));
end

for i = 1:n
    pvec = squeezedTL * w;
    choice = throwDie(pvec);
    seq(1,i) = choice;
    w = tl(:,:,choice) * w;
    w = w / sum(w);    
end

    