function [stateSeq, obsSeq] = genStateObsSeq(tl, w0, n)
% Generates a random state sequence stateSeq from OOM taulist tl, start
% vector w0, of length n; and the corresponding sequence obsSeq of
% observations. The state sequence starts with the first state obtained
% *after* w0.
% StateSeq contains states as columns.
% Should be used only for valid OOMs tl, w0 (which do not give negative
% probs). If the OOM is not known to be valid (typical for learnt OOMs),
% better use compStateSeq_robust instead.
% 
% V 1.0, July 8, 2007
% (c) Herbert Jaeger
%
% Copyright and limitation of liability: as specified in the document
% Copyright.txt that comes along with the OOM toolbox package from which 
% this file is taken.

alphabetsize = size(tl,3);
dim = size(tl,1);
stateSeq = zeros(dim,n);
w = w0;
squeezedTL = zeros(alphabetsize,dim);
for i = 1:alphabetsize
    squeezedTL(i,:) = sum(tl(:,:,i));
end

for i = 1:n
    pvec = squeezedTL * w;
    choice = throwDie(pvec);    
    w = tl(:,:,choice) * w;
    w = w / sum(w);  
    stateSeq(:,i) = w;
    obsSeq(1,i) = choice;
end

    