function stateSeq = genStateSeq(tl, w0, seqArg)
% Generates a state sequence stateSeq from OOM taulist tl and start
% vector w0. If seqArg is an integer n, a random sequence of length n is
% generated; if seqArg is a row vector of observation indices (ie.
% represents an observation sequence), this sequence is used to 
% drive the OOM evolution.  
% StateSeq contains states as columns.
% The starting state w0 is not listed in the outputted stateSeq, which
% instead starts with the first state vector obtained after w0.
% Should be used only for valid OOMs tl, w0 (which do not give negative
% probs). If the OOM is not known to be valid (typical for learnt OOMs),
% better use compStateSeq_robust instead.
% 
% Created July 8, 2007, Herbert Jaeger
% Rev 1, July 13, 2007, H. Jaeger (third argument semantics changed)
%
% Copyright and limitation of liability: as specified in the document
% Copyright.txt that comes along with the OOM toolbox package from which 
% this file is taken.

alphabetsize = size(tl,3);
dim = size(tl,1);
if length(seqArg) == 1
    randomRun = 1;
    n = seqArg;
else
    randomRun = 0;
    n = length(seqArg);
end

stateSeq = zeros(dim,n);
w = w0;

if randomRun
    squeezedTL = zeros(alphabetsize,dim);
    for i = 1:alphabetsize
        squeezedTL(i,:) = sum(tl(:,:,i));
    end
    for i = 1:n
        pvec = squeezedTL * w;
        choice = throwDie(pvec);    
        w = tl(:,:,choice) * w;
        w = w / sum(w);  
        stateSeq(:,i) = w;
    end
else % case seqArg is a driving obs sequence which we use
    for i = 1:n        
        w = tl(:,:,seqArg(i)) * w;
        w = w / sum(w);  
        stateSeq(:,i) = w;
    end
end

