function [tlInt, rho]  = makeInterpretable(tl,C, evLength)
% transforms a taulist tl into OOM taulist that is interpretable with
% respect to a characterizer C
%
% input arguments:
% tl: array of size dim x dim x alphabetsize, containing the observable
%     operators
% C: characterizer matrix, size dim x \kappa
% evLength: length of characteristic sequences
% 
% outputs:
% tlInt: array of size dim x dim x alphabetsize, containing the observable 
%        operators of the resulting interpretable OOM
% rho: the transformation matrix of size dim x dim that transforms tl into
%      tlInt
%
% V 1.0, May 2005
% (c) Herbert Jaeger
%
% Copyright and limitation of liability: as specified in the document
% Copyright.txt that comes along with the OOM toolbox package from which 
% this file is taken.

alphabetsize = size(tl,3);
dim = size(tl,1);
kappa = alphabetsize^evLength;

w0Dummy = [1; zeros(dim - 1,1)];
% we only need PiMat which does not depend on w0
[bigV, PiMat, stateMat] = bigVfromModel(tl,w0Dummy,evLength); 

rho = C * PiMat; 
rhoInv = inv(rho);
tlInt = tl;
for i = 1:alphabetsize
    tlInt(:,:,i) = rho * tl(:,:,i) * rhoInv;
end
