function [] = plotOOMFingerprint(tl, w0, numberOfPoints);
% generates a run of tl and plots  into currently active figure.
% The dim of the OOM must be at least 3. 
% If the dim is larger than 3, a projection on a 3dim subspace is plotted.
% Up to 6 observables are plotted in different colours (b g r c k m). If
% there are more observables, all the remaining after the 6th are plotted
% in colour m. 
%
% V 1.0, May 2005
% (c) Herbert Jaeger
%
% Copyright and limitation of liability: as specified in the document
% Copyright.txt that comes along with the OOM toolbox package from which 
% this file is taken.

alphabetsize = size(tl,3);
dim = size(tl,1);
dim = length(tl(1,:,1));
if dim < 3
    disp('Warning: cannot plot states of dim < 3');
else
    % compute (admittedly arbitrary) matrix for projecting higher-dim states to 3dim    
    projectionMat = zeros(3,dim);
    for d = 1:dim
        d3 = mod(d,3);
        if d3 == 0  d3 = 3; end
        projectionMat(d3,d) = 1;
    end
    % generate run of lenght numberOfPoints, collect states in 2 or 3 lists
    % according to observable producing the state
    statePlotLength = numberOfPoints;
    modStatePL3D = zeros(statePlotLength,3,min([alphabetsize 6]));
    modStateCounters = ones(min([alphabetsize 6]));
    w = w0;
    squeezedTL = zeros(alphabetsize,dim);
    for i = 1:alphabetsize
        squeezedTL(i,:) = sum(tl(:,:,i));
    end
    for n = 1:numberOfPoints
        % update model        
        pvec = squeezedTL * w;
        choice = throwDie(pvec);
        w = tl(:,:,choice) * w;
        w = w / sum(w);
        w3 = projectionMat * w; 
        modStatePL3D(modStateCounters(min([choice 6])), :, min([choice 6])) = w3;
        modStateCounters(min([choice 6])) = modStateCounters(min([choice 6])) + 1; 
    end
    
    % plot states
    colorList = 'bgrckm';
    hold on;
    for obsNr = 1:min([alphabetsize 6])
        plotStates3DColored(modStatePL3D(1:modStateCounters(obsNr)-1,:, obsNr),...
            obsNr, colorList);
    end
    % plot triangle
    plot([-0.5 * sqrt(2)  0.5 * sqrt(2) 0 -0.5 * sqrt(2)],...
        [-(1/3)* sqrt(1.5) -(1/3)* sqrt(1.5) (2/3)* sqrt(1.5) -(1/3)* sqrt(1.5)], 'k');
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [] = plotStates3DColored(statelist3D, obsNr, colorList);
% plots a statelist into currently active figure
% statelist contains 3D states as rows
% obsNr gives index of a particular observation symbol. Different obsNr
% lead to different colors

l = length(statelist3D(:,1));
statelist2D = zeros(l,2);
for i = 1:l
    state3D = statelist3D(i,:);
    if state3D(1,1) + state3D(1,2) == 0
        statelist2D(i,:) =  [0 (2/3)* sqrt(1.5)];
    end
    statelist2D(i,:) = [sqrt(2) * (1 - state3D(1,3)) * ...
            (state3D(1,2)/(state3D(1,1) + state3D(1,2)) - 0.5), ...
            sqrt(1.5) * (state3D(1,3) - (1/3))];     
end
plot(statelist2D(:,1), statelist2D(:,2),  [colorList(obsNr), '.'] );

