function [tlRev, w0Rev] = reverseOOM(tl)
% computes a reverse OOM from a "forward" OOM, as explained in Section 11
% in H. Jaeger et al,
% "Learning Observable Operator Models via the ES Algorithm", in "New
% Directions in Statistical Signal Processing: from Systems to Brains"
% (eds. Simon Haykin, Jose Principe, Terry Sejnowski, John McWhirter), MIT Press 2005. 
%
% input argument:
% tl: array of size dim x dim x alphabetsize, containing the forward OOM
% operators
% 
% outputs:
% tlRev: taulist of reverse OOM, same format as input tl
% w0Rev: its initial state
%
% Version 1.0, May 11, 2005
% Copyright Herbert Jaeger
%
% Copyright and limitation of liability: as specified in the document
% Copyright.txt that comes along with the OOM toolbox package from which 
% this file is taken.

dim = size(tl,1);
alphabetsize = size(tl,3);
w0 = compW0(tl);
rho = dim * diag(w0);
% rho = diag(w0); would work just as well
if min(abs(w0)) < 0.0000001
    disp('Warning: starting state has entry close to 0 in call to reverseOOM. Results may be inaccurate. Try to transform input OOM first s.th. its starting state is all non-zero');
end
rhoInv = inv(rho);
tlRev = tl;
for i = 1:alphabetsize
    tlRev(:,:,i) = rho * (tl(:,:,i))' * rhoInv;
end
w0Rev = w0;